#################################################################################
#                                                                               
#                       copyright (c) 2003 Ingeniweb SARL                       
#                       copyright (c) 2006 Felix Kurth                                                          
#################################################################################

"""I18N Folder main class"""

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
import string

from Products.CMFCore.CMFCorePermissions import View, ManageProperties, ListFolderContents
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFPlone.PloneFolder import PloneFolder
from Products.Archetypes.OrderedBaseFolder import OrderedBaseFolder
from Products.Archetypes.public import *
from Products.ATContentTypes.content.folder import ATFolder as Base
from Products.ATContentTypes.content.document import finalizeATCTSchema

from Products.CMFCore import CMFCorePermissions
from Products.CMFDefault import SkinnedFolder
from AccessControl import Permissions, getSecurityManager, ClassSecurityInfo, Unauthorized
from zExceptions import Redirect 

from Products.CMFPlone.interfaces.Translatable import ITranslatable
from Products.CMFCore.permissions import ModifyPortalContent

from Products.I18NFolder.config import *
from Products.LinguaPlone import permissions
from Products.LinguaPlone.I18NBaseObject import AlreadyTranslated
from Products.CMFCore.utils import getToolByName
from Acquisition import aq_base,aq_inner,aq_parent

schema = Base.schema.copy()
schema['description'].edit_accessor = 'getDescription'
finalizeATCTSchema(schema)


class I18NFolder (Base):
    """Translatable Folder"""
    __implements__ = Base.__implements__ 
    
    portal_type = meta_type = 'I18NFolder'
    archetype_name = 'IFolder'
    content_icon = 'i18nfolder_icon.gif'
    schema = schema
    
    typeDescription= 'A multilingual folder'
    typeDescMsgId  = 'I18NFolder_description_edit'

    default_view = 'folder_contents'
    immediate_view = 'folder_contents'
    #suppl_views = ('ifwdocument_view_preview', 'ifwdocument_view_float', 'folder_listing', 'folder_summary_view', 'folder_tabular_view', 'atct_album_view')

    _at_rename_after_creation = True
    LanguageFixed = None
    security = ClassSecurityInfo()

    def __init__(self, id, title=''):
        """__init__(self, id, title='')"""
        # NOTA : We shouldn't call parent's __init__ method as it would link to PortalFolder.__init__ and this
        # method sets 'self.id' and 'self.title' which is unuseful for us.
        self.id = id
        self.folder_languages = {}
        self.LanguageFixed = None
    
   
    # getCurrentLanguage method
    def getCurrentLanguage(self):
        """
        getCurrentLanguage(self)
        """
        languageTool = getToolByName(self, 'portal_languages', None)
        if languageTool:
            current_language = languageTool.getPreferredLanguage()
        else:
            current_language = 'en'
        return current_language

    # getLanguageObject method. Use fallback
    def getLanguageObject(self, language=None, fallback=True):
        """
        getLanguageObject(self)
        """
        if language is None:
            language = self.getCurrentLanguage()
        #while reindexing....
        if self.LanguageFixed:
            language = self.LanguageFixed
        
        if self.hasTranslation(language):
            return self.folder_languages[language]
        if self.hasTranslation('neutral'):
            return self.folder_languages['neutral']
        if not fallback:
            return None
        language = self.getDefaultLanguage()
        if self.hasTranslation(language):
            return self.folder_languages[language]
        l = self.folder_languages.keys()
        return self.folder_languages[l[0]]
        
    #Overwrite this method, in your dereived classes to use more fields...
    def createLanguageObject(self,lang='neutral'):
        language_object = {'title' : '', 'description' : '','cid' : '', 'lang' : lang}
        return language_object 
    
    # Set title method
    def setTitle(self, title='', language=None):
        """
        setTitle(self, title='', lang_code=None)
        """
        language_object = self.getLanguageObject(language)
        language = language_object['lang']
        language_object['title'] = title
        self.folder_languages[language] = language_object
            
    # Title method
    def Title(self, lang_code=None):
        """
        getTitle(self)
        """
        # Can't return a title if there are no languages in the folder    
        if self.folder_languages:
            language_object = self.getLanguageObject(lang_code)
            title = language_object['title']
        else:
            title = ""        
        return title
    
    # title_or_id method    
    def title_or_id(self, lang_code=None):
        """
        title_or_id(self)
        """
        title = self.Title(lang_code)
        
        if title:
            return title
        else:
            return self.getId()
    
    
    # Set description method
    def setDescription(self, description='', language=None):
        """
        setDescription(self, description='', lang_code=None)
        """
        language_object = self.getLanguageObject(language)
        language = language_object['lang']
        language_object['description'] = description
        self.folder_languages[language] = language_object
        
    # Description method
    def Description(self, lang_code=None):
        """
        getDescription(self)
        """
        language_object = self.getLanguageObject(lang_code)
        description = language_object['description']
        return description
    
    def getDescription(self, lang_code=None):
        return self.Description(lang_code)
    
    def isTranslation(self):
        o = self.getLanguageObject()
        if not o:
            return None
        if o['lang'] == 'neutral':
            return None
        return o['lang']
    
    # getLanguages method. Return a dictionnary of available languages
    def getLanguages(self):
        """
        getLanguages(self)
        """
        languages_list = []
        languages = {}
        
        # Test if portal_languages tool exists
        try:
            languages_list = self.portal_languages.listSupportedLanguages()
        except:
            # So use default languages list
            languages_list = self.availableLanguages()
        
        # Build dictionary    
        for language in languages_list:
            languages[language[0]] = language[1]
        return languages
        
    # getDefaultLanguage method
    def getDefaultLanguage(self):
        """
        getDefaultLanguage(self)
        """
        languageTool = getToolByName(self, 'portal_languages', None)
        if languageTool:
            return languageTool.getDefaultLanguage()
        else:
            return 'en'
        
    # getDefinedLanguages method
    def getDefinedLanguages(self):
        """
        getDefinedLanguages(self)
        Languages wich are already translated-
        """
        return self.getTranslations().keys()
    
        
    # getUndefinedLanguages method
    def getUndefinedLanguages(self):
        """
        getUndefinedLanguages(self)
        """
        
        defined_languages = self.folder_languages.keys()
        undefined_languages = self.getLanguages()
        
        # Delete defined languages from undefined_languages
        for code in defined_languages:
            del(undefined_languages[code])
        return undefined_languages
    
    
    # Add all languages in catalog
    security.declareProtected(View, 'SearchableText')
    def SearchableText(self):
        "Returns a concatination of all searchable text"
        search = []
        language_object = self.getLanguageObject()
        
        if language_object:
            title = language_object['title']
            description = language_object['description']
            if title:
                search.append(title)
            if description:
                search.append(description)
        ret = string.join(search, ' ')
        return ret
    
    security.declareProtected(permissions.View, 'getTranslationLanguages')
    def getTranslationLanguages(self):
        """
        Return a list of language codes.
        """
        return self.getTranslations().keys()        

    security.declareProtected(permissions.View, 'getNonCanonicalTranslations')
    def getNonCanonicalTranslations(self):
        return  self.getTranslations()
        
    def getTranslations(self):
        """
        Return a dict of {lang : [object, wf_state]}
        """
        result = {}
        workflow_tool = getToolByName(self, 'portal_workflow', None)
        state = workflow_tool.getInfoFor(self, 'review_state', None)
        for lang in self.folder_languages:
            if not lang == 'neutral':
                result[lang] = [self, state]            
        return result
    
   
    def removeTranslation(self,lang):
        if self.hasTranslation(lang):
            #The last on..., go to neutral, but dont delete...
            if len(self.folder_languages) == 1:
                self.setLanguage('neutral')
                return
            self.unindexObject(lang)
            del(self.folder_languages[lang])
        
    security.declareProtected(permissions.View, 'isCanonical')
    def isCanonical(self):
        """Returns the canonical translation."""
        return True
    
    
    security.declareProtected(permissions.View, 'getCanonical')
    def getCanonical(self):
        """Returns the canonical translation."""
        return self
    
    security.declareProtected(permissions.View, 'getCanonicalLanguage')
    def getCanonicalLanguage(self):
        """Returns the canonical translation."""
        lo = self.getLanguageObject(self.getDefaultLanguage)
        if lo and lo['lang'] == 'neutral':
            return None
        if lo:
            return lo['lang']
        return None
        
    
    security.declareProtected(permissions.View, 'getTranslation')
    def getTranslation(self, language=None):
        if language == None:
            return self
        if self.hasTranslation(language):
            return self
        return None
        

    security.declareProtected(permissions.AddPortalContent, 'addTranslation')
    def addTranslation(self, language, *args, **kwargs):
        """Adds a translation."""
        if self.hasTranslation(language):
            return
        if language == 'neutral':
            return
        if self.hasTranslation('neutral'):
            self.setLanguage(language,'neutral')
        else:   
            self.folder_languages[language] = self.createLanguageObject(language)
            self.reindexObject()
        
    security.declareProtected(permissions.View, 'hasTranslation')
    def hasTranslation(self,language):
        return self.folder_languages and self.folder_languages.has_key(language)
        
        
    security.declareProtected(permissions.View, 'getLanguage')
    def getLanguage(self):
        """Returns the language code."""
        return self.Language()
    
    def setLanguage(self,language,current=None):
        """
        Sets the language for the current translation 
        """
        if language=='':
            language = 'neutral'
        if current is None:
            currentobj = self.getLanguageObject(current,False)
            if not currentobj:
                return
            current = currentobj['lang']
        #we are called with an explicit current lang, but its not here.    
        if not self.hasTranslation(current):
            return
        if self.hasTranslation(language):
            return
        if current== language:
            return
        
        new_lang_object = currentobj.copy()
        new_lang_object['lang'] = language
        self.folder_languages[language] = new_lang_object
        self.removeTranslation(current)
        #clean completely up if we change to neutral
        if language=='neutral':
            trans = self.getTranslationLanguages()
            for t in trans:
                self.removeTranslation(t)
        self.reindexObject()
        return

    def Language(self):
        #The catalog ask us while reindexing... 
        if self.LanguageFixed:
            if self.LanguageFixed=='neutral':
                return ''
            return self.LanguageFixed
        o = self.getLanguageObject()
        if o['lang'] == 'neutral':
            return ''
        return o['lang']
    
   
    security.declareProtected(ModifyPortalContent, 'indexObject')
    def indexObject(self):
        """
            Index the object in the portal catalog.
        """
        self.reindexObject()
        
    security.declareProtected(ModifyPortalContent, 'unindexObject')
    def unindexObject(self, lang=None):
        """
            Unindex the object from the portal catalog.
        """
        catalog = self._getCatalogTool()
        if catalog is not None:
                if lang:
                    uid = self.folder_languages[lang]['cid']
                    if uid:
                        catalog.uncatalog_object(uid)
                else:
                    langs = self.folder_languages.keys()
                    for lang in langs:
                        uid = self.folder_languages[lang]['cid']
                        if uid:
                            catalog.uncatalog_object(uid)

 
    
    security.declareProtected(ModifyPortalContent, 'reindexObject')
    def reindexObject(self, idxs=[], update_metadata=1):
        """
            Reindex the object in the portal catalog.
            If idxs is present, only those indexes are reindexed.
            The metadata is always updated.

            Also update the modification date of the object,
            unless specific indexes were requested.
            
            We create an entry for every language!! This is possible
            because the AbstractCatalogBrain,getPath() Method
            is patched. 
            See patches.py
        """
        if idxs == []:
            # Update the modification date.
            if hasattr(aq_base(self), 'notifyModified'):
                self.notifyModified()
        catalog = self._getCatalogTool()
        if catalog is not None:
            #uid = self.getPhysicalPath()
            uid = self.absolute_url_path()

            #we clean completley up
            #this is neccessary because in the case of move-actions
            if idxs == [] :
                self.unindexObject()
            #catalog.uncatalog_object(uid)
            
            for lang in self.folder_languages:
                if not lang=='neutral':
                    self.LanguageFixed = lang
                    id = uid + '--i18nfolder-' +lang
                    catalog.reindexObject(self, idxs, update_metadata, id)
                else:
                    id = uid
                    self.LanguageFixed = 'neutral'
                    #self.REQUEST.set('LANGUAGE', '')
                    catalog.reindexObject(self, idxs, update_metadata, id)
                #safe this for cleaning, never play with this id... your portal_catalog messes up.
                self.folder_languages[lang]['cid'] = id
            
            self.LanguageFixed  = None
            
        
    security.declarePrivate('manage_afterClone')
    def manage_afterClone(self, item):
        Base.manage_afterClone(self, item)
        self.reindexObject()
        #Dont Reindex here
         
    def manage_afterAdd(self, item, container):
        """
            Add self to the catalog.
            (Called when the object is created or moved.)
        """
        if not self.folder_languages:
            self.folder_languages['neutral'] = self.createLanguageObject('neutral')
        self.indexObject()
        Base.manage_afterAdd(self, item, container)
        
    
registerType(I18NFolder)

