#patches
import Acquisition, Record
from ZODB.POSException import ConflictError
# Switch for new behavior, raise exception instead of returning None.
# Use 'catalog-getObject-raises off' in zope.conf to restore old behavior.
GETOBJECT_RAISES = True

from Products.CMFPlone.utils import safe_callable
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.permissions import AccessContentsInformation, \
                        ManagePortal, ManageUsers, ModifyPortalContent, View

from Products.ZCatalog.CatalogBrains import AbstractCatalogBrain


def _getPath(self):
    """Get the physical path for this record"""
    #Use the path-index if its there
    pi = None
    try:
        #if the catalog is messed up, this crashes.
        pi = self.aq_parent.getIndexDataForRID(self.data_record_id_)
    except:
        pass
    if pi and pi.has_key('path'):
        return pi['path']
    #fallback for catalogs without pathindex
    return self.aq_parent.getpath(self.data_record_id_)

AbstractCatalogBrain.getPath = _getPath

from Products.CMFPlone.PloneTool import PloneTool

def _reindexOnReorder(self, parent):
        """ Catalog ordering support """

        # For now we will just reindex all objects in the folder. Later we may
        # optimize to only reindex the objs that got moved. Ordering is more
        # for humans than machines, therefore the fact that this won't scale
        # well for btrees isn't a huge issue, since btrees are more for
        # machines than humans.
        mtool = getToolByName(self, 'portal_membership')
        if not mtool.checkPermission(ModifyPortalContent, parent):
            return
        cat = getToolByName(self, 'portal_catalog')
        cataloged_objs = cat(path = {'query':'/'.join(parent.getPhysicalPath()),
                                     'depth': 1})
        for brain in cataloged_objs:
            obj = brain.getObject()
            # Don't crash when the catalog has contains a stale entry
            if obj is not None:
                reindexObject = getattr(obj, 'reindexObject', None)
                if reindexObject and safe_callable(reindexObject):
                    obj.reindexObject(['getObjPositionInParent'])
                else:
                    cat.reindexObject(obj,['getObjPositionInParent'],update_metadata=0)
            else:
                # Perhaps we should remove the bad entry as well?
                log('Object in catalog no longer exists, cannot reindex: %s.'%
                                    brain.getPath())

PloneTool.reindexOnReorder = _reindexOnReorder

from BTrees.IIBTree import IISet, intersection, union, multiunion
from BTrees.OOBTree import OOBTree
from BTrees.OIBTree import OIBTree
from zLOG import LOG, ERROR
from Products.PluginIndexes.common.util import parseIndexRequest
from Products.PluginIndexes.common import safe_callable

from Products.ExtendedPathIndex.ExtendedPathIndex import ExtendedPathIndex
def _search(self, path, default_level=0, depth=-1, navtree=0,
                                                          navtree_start=0):
     """
     path is either a string representing a
     relative URL or a part of a relative URL or
     a tuple (path,level).

     level >= 0  starts searching at the given level
     level <  0  not implemented yet
     """

     if isinstance(path, str):
         startlevel = default_level
     else:
         startlevel = int(path[1])
         path = path[0]

     absolute_path = isinstance(path, basestring) and path.startswith('/')

     comps = filter(None, path.split('/'))

     orig_comps = [''] + comps[:]
     # Optimization - avoid using the root set
     # as it is common for all objects anyway and add overhead
     # There is an assumption about catalog/index having
     # the same container as content
     if default_level == 0:
         indexpath = list(filter(None, self.getPhysicalPath()))
         while min(len(indexpath), len(comps)):
             if indexpath[0] == comps[0]:
                 del indexpath[0]
                 del comps[0]
                 startlevel += 1
             else:
                 break

     if len(comps) == 0:
         if depth == -1 and not navtree:
             return IISet(self._unindex.keys())

     # Make sure that we get depth = 1 if in navtree mode
     # unless specified otherwise

     orig_depth = depth
     if depth == -1:
         depth = 0 or navtree

     # Optimized navtree starting with absolute path
     if absolute_path and navtree and depth == 1 and default_level==0:
         set_list = []
         # Insert root element
         if navtree_start >= len(orig_comps):
             navtree_start = 0
         # create a set of parent paths to search
         for i in range(len(orig_comps), navtree_start, -1):
             parent_path = '/'.join(orig_comps[:i])
             parent_path = parent_path and parent_path or '/'
             try:
                 set_list.append(self._index_parents[parent_path])
             except KeyError:
                 pass
         return multiunion(set_list)
     # Optimized breadcrumbs
     elif absolute_path and navtree and depth == 0 and default_level==0:
         item_list = []
         tmp_set = IISet()
         # Insert root element
         if navtree_start >= len(orig_comps):
             navtree_start = 0
         # create a set of parent paths to search
         for i in range(len(orig_comps), navtree_start, -1):
             parent_path = '/'.join(orig_comps[:i])
             parent_path = parent_path and parent_path or '/'
             try:
                 orig_path = '/'.join(orig_comps[:i+1])
                 tmp_set = self._index_parents[parent_path]
                 item_list.append(IISet([id for id in tmp_set if self.getEntryForObject(id) == orig_path]))
             except KeyError:
                 pass
         return multiunion(item_list)
     # Specific object search
     elif absolute_path and orig_depth == 0 and default_level == 0:
         try:
             return IISet([self._index_items[path]])
         except KeyError:
             return IISet()
     # Single depth search
     elif absolute_path and orig_depth == 1 and default_level == 0:
         # only get objects contained in requested folder
         try:
             return self._index_parents[path]
         except KeyError:
             return IISet()
     # Sitemaps, relative paths, and depth queries
     elif startlevel >= 0:

         pathset = None # Same as pathindex
         navset  = None # For collecting siblings along the way
         depthset = None # For limiting depth

         if navtree and depth and \
                self._index.has_key(None) and \
                self._index[None].has_key(startlevel):
             navset = self._index[None][startlevel]

         for level in range(startlevel, startlevel+len(comps) + depth):
             if level-startlevel < len(comps):
                 comp = comps[level-startlevel]
                 if not self._index.has_key(comp) or not self._index[comp].has_key(level): 
                     # Navtree is inverse, keep going even for
                     # nonexisting paths
                     if navtree:
                         pathset = IISet()
                     else:
                         return IISet()
                 else:
                     pathset = intersection(pathset,
                                                  self._index[comp][level])
                 if navtree and depth and \
                        self._index.has_key(None) and \
                        self._index[None].has_key(level+depth):
                     navset  = union(navset, intersection(pathset,
                                           self._index[None][level+depth]))
             if level-startlevel >= len(comps) or navtree:
                 if self._index.has_key(None) and self._index[None].has_key(level):
                     depthset = union(depthset, intersection(pathset,
                                                 self._index[None][level]))

         if navtree:
             return union(depthset, navset) or IISet()
         elif depth:
             return depthset or IISet()
         else:
             return pathset or IISet()

     else:
         results = IISet()
         for level in range(0,self._depth + 1):
             ids = None
             error = 0
             for cn in range(0,len(comps)):
                 comp = comps[cn]
                 try:
                     ids = intersection(ids,self._index[comp][level+cn])
                 except KeyError:
                     error = 1
             if error==0:
                 results = union(results,ids)
         return results

ExtendedPathIndex.search = _search



#This function is patched to call obj.indexObject instead of catalog.catalog_object(object)
from Products.ZCatalog import ZCatalog
def _ZopeFindAndApply(self, obj, obj_ids=None, obj_metatypes=None,
                         obj_searchterm=None, obj_expr=None,
                         obj_mtime=None, obj_mspec=None,
                         obj_permission=None, obj_roles=None,
                         search_sub=0,
                         REQUEST=None, result=None, pre='',
                         apply_func=None, apply_path=''):
        """Zope Find interface and apply

        This is a *great* hack.  Zope find just doesn't do what we
        need here; the ability to apply a method to all the objects
        *as they're found* and the need to pass the object's path into
        that method.

        """

        if result is None:
            result=[]

            if obj_metatypes and 'all' in obj_metatypes:
                obj_metatypes=None

            if obj_mtime and type(obj_mtime)==type('s'):
                obj_mtime=DateTime(obj_mtime).timeTime()

            if obj_permission:
                obj_permission=p_name(obj_permission)

            if obj_roles and type(obj_roles) is type('s'):
                obj_roles=[obj_roles]

            if obj_expr:
                # Setup expr machinations
                md=td()
                obj_expr=(Eval(obj_expr), md, md._push, md._pop)

        base=obj
        if hasattr(obj, 'aq_base'):
            base=obj.aq_base

        if not hasattr(base, 'objectItems'):
            return result
        try:    items=obj.objectItems()
        except: return result

        try: add_result=result.append
        except:
            raise AttributeError, `result`

        for id, ob in items:
            if pre: p="%s/%s" % (pre, id)
            else:   p=id

            dflag=0
            if hasattr(ob, '_p_changed') and (ob._p_changed == None):
                dflag=1

            if hasattr(ob, 'aq_base'):
                bs=ob.aq_base
            else: bs=ob

            if (
                (not obj_ids or absattr(bs.id) in obj_ids)
                and
                (not obj_metatypes or (hasattr(bs, 'meta_type') and
                 bs.meta_type in obj_metatypes))
                and
                (not obj_searchterm or
                 (hasattr(ob, 'PrincipiaSearchSource') and
                  ob.PrincipiaSearchSource().find(obj_searchterm) >= 0
                  ))
                and
                (not obj_expr or expr_match(ob, obj_expr))
                and
                (not obj_mtime or mtime_match(ob, obj_mtime, obj_mspec))
                and
                ( (not obj_permission or not obj_roles) or \
                   role_match(ob, obj_permission, obj_roles)
                )
                ):
                #This is the added stuff
                if hasattr(ob, 'indexObject'):
                    ob.indexObject()
                #End off added stuff
                elif apply_func:
                    apply_func(ob, (apply_path+'/'+p))
                else:
                    add_result((p, ob))
                    dflag=0

            if search_sub and hasattr(bs, 'objectItems'):
                self.ZopeFindAndApply(ob, obj_ids, obj_metatypes,
                                      obj_searchterm, obj_expr,
                                      obj_mtime, obj_mspec,
                                      obj_permission, obj_roles,
                                      search_sub,
                                      REQUEST, result, p,
                                      apply_func, apply_path)
            if dflag: ob._p_deactivate()

        return result
ZCatalog.ZopeFindAndApply = _ZopeFindAndApply